Stan Math Library: stan/math/rev/mat/fun/log_determinant.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

log_determinant.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_REV_MAT_FUN_LOG_DETERMINANT_HPP
2 #define STAN_MATH_REV_MAT_FUN_LOG_DETERMINANT_HPP
3
4 #include <stan/math/prim/mat/fun/Eigen.hpp>
5 #include <stan/math/prim/mat/err/check_square.hpp>
6 #include <stan/math/rev/core.hpp>
7
8 namespace stan {
9
10 namespace math {
11
12 template <int R, int C>
13 inline var log_determinant(const Eigen::Matrix<var, R, C>& m) {
14 using Eigen::Matrix;
15
16 math::check_square("log_determinant", "m", m);
17
18 Matrix<double, R, C> m_d(m.rows(), m.cols());
19 for (int i = 0; i < m.size(); ++i)
20 m_d(i) = m(i).val();
21
22 Eigen::FullPivHouseholderQR<Matrix<double, R, C> > hh
23 = m_d.fullPivHouseholderQr();
24
25 double val = hh.logAbsDeterminant();
26
27 vari** varis
28 = ChainableStack::memalloc_.alloc_array<vari*>(m.size());
29 for (int i = 0; i < m.size(); ++i)
30 varis[i] = m(i).vi_;
31
32 Matrix<double, R, C> m_inv_transpose = hh.inverse().transpose();
33 double* gradients
34 = ChainableStack::memalloc_.alloc_array<double>(m.size());
35 for (int i = 0; i < m.size(); ++i)
36 gradients[i] = m_inv_transpose(i);
37
38 return var(new precomputed_gradients_vari(val, m.size(),
39 varis, gradients));
40 }
41
42 }
43 }
44 #endif
core.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::AutodiffStackStorage::memalloc_
static stack_alloc memalloc_
Definition: autodiffstackstorage.hpp:16
stan::math::vari
The variable implementation base class.
Definition: vari.hpp:30
stan::math::log_determinant
fvar< T > log_determinant(const Eigen::Matrix< fvar< T >, R, C > &m)
Definition: log_determinant.hpp:20
stan::math::var
Independent (input) and dependent (output) variables for gradients.
Definition: var.hpp:31
stan::math::precomputed_gradients_vari
A variable implementation taking a sequence of operands and partial derivatives with respect to the o...
Definition: precomputed_gradients.hpp:21
Eigen.hpp
stan::math::stack_alloc::alloc_array
T * alloc_array(size_t n)
Allocate an array on the arena of the specified size to hold values of the specified template paramet...
Definition: stack_alloc.hpp:186
stan::math::check_square
bool check_square(const char *function, const char *name, const Eigen::Matrix< T_y, Eigen::Dynamic, Eigen::Dynamic > &y)
Return true if the specified matrix is square.
Definition: check_square.hpp:28
check_square.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.